==> ssp_api-1.2.0.0-sources.jar: com/cloud/hisavana/sdk/internal/native_/TranNative.java <==
package com.cloud.hisavana.sdk.internal.native_;

import android.content.Context;
import android.support.annotation.NonNull;
import android.view.View;
import android.view.ViewGroup;

import com.cloud.hisavana.sdk.ad.base.AdxAdFactory;
import com.cloud.hisavana.sdk.ad.base.PlatformUtil;
import com.cloud.hisavana.sdk.api.config.AdsConfig;
import com.cloud.hisavana.sdk.api.view.AAdChoicesView;
import com.cloud.hisavana.sdk.api.view.SspNativeView;
import com.cloud.hisavana.sdk.common.adapter.INative;
import com.cloud.hisavana.sdk.common.bean.TaNativeInfo;
import com.cloud.hisavana.sdk.common.callback.INativeBridge;
import com.cloud.hisavana.sdk.common.constant.Constants;
import com.cloud.hisavana.sdk.common.tranmeasure.MeasureSession;
import com.cloud.hisavana.sdk.common.tranmeasure.MeasureSessionManager;
import com.cloud.hisavana.sdk.common.util.AdLogUtil;
import com.cloud.hisavana.sdk.common.util.EmptyUtil;
import com.cloud.hisavana.sdk.common.util.Preconditions;
import com.cloud.hisavana.sdk.data.bean.response.AdsDTO;
import com.cloud.hisavana.sdk.internal.base.BaseAd;

import java.util.List;

/**
 * Created by jianbing.yu on 2019/2/25.
 */
public class TranNative extends BaseAd implements INativeBridge {

    private NativeGemini mPlatform;

    private List<AdsDTO> mAdBeans;
    private INative mNative;

    private boolean isLoading;
    private boolean unifiedCallback = true;
    //20210923 imageurl加载不成功时候，不显示native广告，所以把imageurl和icon的加载分开
    private int fillType = AdsConfig.FillType.IMAGE;


    // =============================================================================================


    private CallbackStub mStub = new CallbackStub() {

        /**
         * 网络请求 成功回调
         */
        @Override
        protected void onAdResponse(List<AdsDTO> adBeanList) {
            destroyMeasureSessions();
            mAdBeans = adBeanList;
            loadPlatformAd();
        }

        @Override
        protected void onAdTimeout() {
            if (mPlatform != null && EmptyUtil.isCollectionNotEmpty(mPlatform.getNativeInfos())) {
                if (mPlatform.getNativeInfos().size() > 1) {
                    mInternalListener.onAdLoaded(mPlatform.getNativeInfos());
                } else {
                    setOneByOne(false);
                    mInternalListener.onAdLoaded(mPlatform.getNativeInfos());
                }
            } else {
                super.onAdTimeout();
            }
        }

        @Override
        protected void onAdLoaded(List<TaNativeInfo> nativeInfos) {
            for (TaNativeInfo nativeInfo : nativeInfos) {
                if (nativeInfo != null) {
                    nativeInfo.setBidPrice(mAdBeans != null && mAdBeans.size() > 0  && mAdBeans.get(0) != null
                            ? mAdBeans.get(0).getBidPrice() : 0);
                    nativeInfo.nativeBridge = TranNative.this;
                }
            }
        }
    };

    private MeasureSession.AdImpressionListener impressionListener = new MeasureSession.AdImpressionListener<AdsDTO>() {
        @Override
        public void onViewImpressed(AdsDTO nativeInfo) {
            if (mPlatform != null && nativeInfo != null) {
                mPlatform.adImpression(null);
            }
        }
    };

    /**
     * 获取广告item
     */
    private void loadPlatformAd() {
        AdLogUtil.LOG.d("loadPlatformAd start load ad");
        boolean isResetTimer = mPlatform.loadAd();
        if (isResetTimer) {
            resetTimerTask();
            runTimerTask();
        }
    }


    // =============================================================================================


    public TranNative(String placementId) {
        super(Constants.AdType.NATIVE, placementId);
        mNative = AdxAdFactory.newInstance().getNative(placementId);
        mNative.setAdListener(mInternalListener);
        mPlatform = new NativeGemini(this);
    }


    // =============================================================================================


    @Override
    public void setPlacementId(String placementId) {
        this.mPlacementId = placementId;
        mNative.setPlacementId(placementId);
    }

    @Override
    protected boolean loadPolyAd() {
        return isLoading = mNative.loadAd(mRequestId);
    }

    @Override
    protected CallbackStub callbackStub() {
        return mStub;
    }

    @Override
    public void destroy() {
        Preconditions.runOnMainThread(new Preconditions.Callback() {
            @Override
            public void onRun() {
                destroyMeasureSessions();
                mNative.destroy();
                TranNative.super.destroy();
                mPlatform.destroy();
            }
        });
    }

    @Override
    public int getAdType() {
        return Constants.AdType.NATIVE;
    }

    @Override
    public void onNativeInfoDestroy(final TaNativeInfo nativeInfo) {
        Preconditions.runOnMainThread(new Preconditions.Callback() {
            @Override
            public void onRun() {
                destroySingleMeasureSession(nativeInfo);
                unregisterViews(nativeInfo);
            }
        });
    }

    @Override
    public boolean isAdValid(TaNativeInfo nativeInfo) {
        AdsDTO adBean = PlatformUtil.getAdBean(nativeInfo, mAdBeans);
        return AdsConfig.isAdValid(adBean);
    }

    private void unregisterViews(TaNativeInfo nativeInfo) {
        if (mPlatform != null) {
            mPlatform.unregisterView(nativeInfo);
        }
    }


    // =============================================================================================


    public void setAdCount(int adCount) {
        mNative.setAdCount(adCount);
    }

    public void downloadMedia(final TaNativeInfo ad) {
        Preconditions.runOnMainThread(new Preconditions.Callback() {
            @Override
            public void onRun() {
                mPlatform.downloadMedia(ad);
            }
        });
    }

    public boolean isLoading() {
        return isLoading;
    }

    public AAdChoicesView getAdChoicesView(Context context, TaNativeInfo taNativeInfo) {
        return mNative.getAdChoicesView(context, PlatformUtil.getAdBean(taNativeInfo, mAdBeans));
    }

    List<AdsDTO> adBeans() {
        return mAdBeans;
    }

    INative polyNative() {
        return mNative;
    }

    void setLoading(boolean loading) {
        isLoading = loading;
    }

    boolean isUnifiedCallback() {
        return unifiedCallback;
    }

    int getFillType() {
        return fillType;
    }

    void loadFinished() {
        isLoading = false;
        onLoaded();
    }


    // =============================================================================================


    /**
     * 注册View
     *
     * @param view       暴露给媒体的容器 SspNativeView
     * @param adContains 广告包含的 Views
     * @param adInfo     广告详情信息
     */
    public void registerViews(@NonNull ViewGroup view, List<View> adContains, TaNativeInfo adInfo) {
        Preconditions.checkIsOnMainThread();
        if (mPlatform != null && adInfo != null) {
            TaNativeInfo nativeInfo = adInfo;
            AdsDTO adBean = PlatformUtil.getAdBean(nativeInfo, mAdBeans);
            if (adBean == null) {
                AdLogUtil.LOG.e("adItem is null");
                return;
            }
            if (!mLoaded) {
                AdLogUtil.LOG.e("Ad not loaded.");
                return;
            }
            //显示view
            setupViews(view, adInfo, mNative.getAdChoicesView(view.getContext(), PlatformUtil.getAdBean(adInfo, mAdBeans)));
            boolean isAdValid = isAdValid(nativeInfo);
            //打底广告是否有效，是看打底广告的有效期
            if (isDefaultAd) isAdValid = true;
            // 广告有效就测量
            doImpressionLogic(view, adBean,isAdValid);

            mPlatform.registerViewForInteraction(view, adContains, nativeInfo, adBean);
            AdLogUtil.LOG.d("Native Ad start registered");
            if (View.GONE == view.getVisibility()) {
                view.setVisibility(View.VISIBLE);
            }
        }
    }

    private void setupViews(@NonNull View view, TaNativeInfo adInfo, AAdChoicesView adChoicesView) {
        if (view != null && view instanceof SspNativeView) {
            ((SspNativeView) view).setupViews(adInfo, adChoicesView);
        } else if (view != null && view instanceof ViewGroup) {
            View tmp;
            for (int index = 0; index < ((ViewGroup) view).getChildCount(); index++) {
                tmp = ((ViewGroup) view).getChildAt(index);
                if (tmp != null && tmp instanceof SspNativeView) {
                    ((SspNativeView) tmp).setupViews(adInfo, adChoicesView);
                    break;
                } else if (tmp != null && tmp instanceof ViewGroup) {
                    setupViews(tmp, adInfo, adChoicesView);
                }
            }
        }
    }

    private void doImpressionLogic(@NonNull View view, AdsDTO adBean,boolean isAdValid) {
        AdLogUtil.LOG.d("current native did not showed...");
        MeasureSession session = MeasureSessionManager.getInstance().getMeasureSession(adBean);
        session.setAdValid(isAdValid);
        session.registerView(view, impressionListener);
    }


    // =============================================================================================


    private void destroyMeasureSessions() {
        if (mPlatform != null && mPlatform.getNativeInfos() != null) {
            for (TaNativeInfo nativeInfo : mPlatform.getNativeInfos()) {
                destroySingleMeasureSession(nativeInfo);
            }
        }
    }

    private void destroySingleMeasureSession(TaNativeInfo nativeInfo) {
        if (nativeInfo != null) {
            // TODO 注销 ===================
            AdsDTO adBean = PlatformUtil.getAdBean(nativeInfo, mAdBeans);
            MeasureSessionManager.getInstance().destroySession(adBean);
        }
    }

}

==> ssp_api-1.2.1.2-sources.jar: com/cloud/hisavana/sdk/internal/native_/TranNative.java <==
package com.cloud.hisavana.sdk.internal.native_;

import android.content.Context;
import android.support.annotation.NonNull;
import android.view.View;
import android.view.ViewGroup;

import com.cloud.hisavana.sdk.ad.AdxNative;
import com.cloud.hisavana.sdk.ad.base.PlatformUtil;
import com.cloud.hisavana.sdk.api.config.AdsConfig;
import com.cloud.hisavana.sdk.api.view.AAdChoicesView;
import com.cloud.hisavana.sdk.api.view.SspNativeView;
import com.cloud.hisavana.sdk.common.adapter.INative;
import com.cloud.hisavana.sdk.common.bean.TaNativeInfo;
import com.cloud.hisavana.sdk.common.callback.INativeBridge;
import com.cloud.hisavana.sdk.common.constant.Constants;
import com.cloud.hisavana.sdk.common.tranmeasure.MeasureSession;
import com.cloud.hisavana.sdk.common.tranmeasure.MeasureSessionManager;
import com.cloud.hisavana.sdk.common.util.AdLogUtil;
import com.cloud.hisavana.sdk.common.util.EmptyUtil;
import com.cloud.hisavana.sdk.common.util.Preconditions;
import com.cloud.hisavana.sdk.data.bean.response.AdsDTO;
import com.cloud.hisavana.sdk.internal.base.BaseAd;

import java.util.List;

/**
 * Created by jianbing.yu on 2019/2/25.
 */
public class TranNative extends BaseAd implements INativeBridge {

    private NativeGemini mPlatform;

    private List<AdsDTO> mAdBeans;
    private INative mNative;

    private boolean isLoading;
    private boolean unifiedCallback = true;
    //20210923 imageurl加载不成功时候，不显示native广告，所以把imageurl和icon的加载分开
    private int fillType = AdsConfig.FillType.IMAGE;


    // =============================================================================================


    /**
     * 网络请求 成功回调
     */
    @Override
    protected void onAdResponseStub(List<AdsDTO> adBeanList) {
        destroyMeasureSessions();
        mAdBeans = adBeanList;
        loadPlatformAd();
    }

    @Override
    protected void onAdTimeoutStub() {
        if (mPlatform != null && EmptyUtil.isCollectionNotEmpty(mPlatform.getNativeInfos())) {
            if (mPlatform.getNativeInfos().size() > 1) {
                mInternalListener.onAdLoaded(mPlatform.getNativeInfos());
            } else {
                setOneByOne(false);
                mInternalListener.onAdLoaded(mPlatform.getNativeInfos());
            }
        } else {
            super.onAdTimeoutStub();
        }
    }

    @Override
    protected void onAdLoadedStub(List<TaNativeInfo> nativeInfos) {
        for (TaNativeInfo nativeInfo : nativeInfos) {
            if (nativeInfo != null) {
                nativeInfo.setBidPrice(mAdBeans != null && mAdBeans.size() > 0  && mAdBeans.get(0) != null
                        ? mAdBeans.get(0).getBidPrice() : 0);
                nativeInfo.nativeBridge = TranNative.this;
            }
        }
    }


    // =============================================================================================


    private MeasureSession.AdImpressionListener impressionListener = new MeasureSession.AdImpressionListener<AdsDTO>() {
        @Override
        public void onViewImpressed(AdsDTO nativeInfo) {
            if (mPlatform != null && nativeInfo != null) {
                mPlatform.adImpression(null);
            }
        }
    };

    /**
     * 获取广告item
     */
    private void loadPlatformAd() {
        AdLogUtil.LOG.d("loadPlatformAd start load ad");
        boolean isResetTimer = mPlatform.loadAd();
        if (isResetTimer) {
            resetTimerTask();
            runTimerTask();
        }
    }


    // =============================================================================================


    public TranNative(String placementId) {
        super(Constants.AdType.NATIVE, placementId);
        mNative = new AdxNative(placementId);
        mNative.setAdListener(mInternalListener);
        mPlatform = new NativeGemini(this);
    }


    // =============================================================================================


    @Override
    public void setPlacementId(String placementId) {
        this.mPlacementId = placementId;
        mNative.setPlacementId(placementId);
    }

    @Override
    protected boolean loadPolyAd() {
        return isLoading = mNative.loadAd(mRequestId);
    }

    @Override
    public void destroy() {
        Preconditions.runOnMainThread(new Preconditions.Callback() {
            @Override
            public void onRun() {
                destroyMeasureSessions();
                mNative.destroy();
                TranNative.super.destroy();
                mPlatform.destroy();
            }
        });
    }

    @Override
    public int getAdType() {
        return Constants.AdType.NATIVE;
    }

    @Override
    public void onNativeInfoDestroy(final TaNativeInfo nativeInfo) {
        Preconditions.runOnMainThread(new Preconditions.Callback() {
            @Override
            public void onRun() {
                destroySingleMeasureSession(nativeInfo);
                unregisterViews(nativeInfo);
            }
        });
    }

    @Override
    public boolean isAdValid(TaNativeInfo nativeInfo) {
        AdsDTO adBean = PlatformUtil.getAdBean(nativeInfo, mAdBeans);
        return AdsConfig.isAdValid(adBean);
    }

    private void unregisterViews(TaNativeInfo nativeInfo) {
        if (mPlatform != null) {
            mPlatform.unregisterView(nativeInfo);
        }
    }


    // =============================================================================================


    public void setAdCount(int adCount) {
        mNative.setAdCount(adCount);
    }

    public void downloadMedia(final TaNativeInfo ad) {
        Preconditions.runOnMainThread(new Preconditions.Callback() {
            @Override
            public void onRun() {
                mPlatform.downloadMedia(ad);
            }
        });
    }

    public boolean isLoading() {
        return isLoading;
    }

    public AAdChoicesView getAdChoicesView(Context context, TaNativeInfo taNativeInfo) {
        return mNative.getAdChoicesView(context, PlatformUtil.getAdBean(taNativeInfo, mAdBeans));
    }

    List<AdsDTO> adBeans() {
        return mAdBeans;
    }

    INative polyNative() {
        return mNative;
    }

    void setLoading(boolean loading) {
        isLoading = loading;
    }

    boolean isUnifiedCallback() {
        return unifiedCallback;
    }

    int getFillType() {
        return fillType;
    }

    void loadFinished() {
        isLoading = false;
        onLoaded();
    }


    // =============================================================================================


    /**
     * 注册View
     *
     * @param view       暴露给媒体的容器 SspNativeView
     * @param adContains 广告包含的 Views
     * @param adInfo     广告详情信息
     */
    public void registerViews(@NonNull ViewGroup view, List<View> adContains, TaNativeInfo adInfo) {
        Preconditions.checkIsOnMainThread();
        if (mPlatform != null && adInfo != null) {
            TaNativeInfo nativeInfo = adInfo;
            AdsDTO adBean = PlatformUtil.getAdBean(nativeInfo, mAdBeans);
            if (adBean == null) {
                AdLogUtil.LOG.e("adItem is null");
                return;
            }
            if (!mLoaded) {
                AdLogUtil.LOG.e("Ad not loaded.");
                return;
            }
            //显示view
            setupViews(view, adInfo, mNative.getAdChoicesView(view.getContext(), PlatformUtil.getAdBean(adInfo, mAdBeans)));
            boolean isAdValid = isAdValid(nativeInfo);
            //打底广告是否有效，是看打底广告的有效期
            if (isDefaultAd) isAdValid = true;
            // 广告有效就测量
            doImpressionLogic(view, adBean,isAdValid);

            mPlatform.registerViewForInteraction(view, adContains, nativeInfo, adBean);
            AdLogUtil.LOG.d("Native Ad start registered");
            if (View.GONE == view.getVisibility()) {
                view.setVisibility(View.VISIBLE);
            }
        }
    }

    private void setupViews(@NonNull View view, TaNativeInfo adInfo, AAdChoicesView adChoicesView) {
        if (view != null && view instanceof SspNativeView) {
            ((SspNativeView) view).setupViews(adInfo, adChoicesView);
        } else if (view != null && view instanceof ViewGroup) {
            View tmp;
            for (int index = 0; index < ((ViewGroup) view).getChildCount(); index++) {
                tmp = ((ViewGroup) view).getChildAt(index);
                if (tmp != null && tmp instanceof SspNativeView) {
                    ((SspNativeView) tmp).setupViews(adInfo, adChoicesView);
                    break;
                } else if (tmp != null && tmp instanceof ViewGroup) {
                    setupViews(tmp, adInfo, adChoicesView);
                }
            }
        }
    }

    private void doImpressionLogic(@NonNull View view, AdsDTO adBean,boolean isAdValid) {
        AdLogUtil.LOG.d("current native did not showed...");
        MeasureSession session = MeasureSessionManager.getInstance().getMeasureSession(adBean);
        session.setAdValid(isAdValid);
        session.registerView(view, impressionListener);
    }


    // =============================================================================================


    private void destroyMeasureSessions() {
        if (mPlatform != null && mPlatform.getNativeInfos() != null) {
            for (TaNativeInfo nativeInfo : mPlatform.getNativeInfos()) {
                destroySingleMeasureSession(nativeInfo);
            }
        }
    }

    private void destroySingleMeasureSession(TaNativeInfo nativeInfo) {
        if (nativeInfo != null) {
            AdsDTO adBean = PlatformUtil.getAdBean(nativeInfo, mAdBeans);
            MeasureSessionManager.getInstance().destroySession(adBean);
        }
    }

}
